package cn.com.infosec.crypto.modes;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.params.ParametersWithIV;

public class OFBBlockCipher implements BlockCipher {
   private byte[] IV;
   private byte[] ofbV;
   private byte[] ofbOutV;
   private final int blockSize;
   private final BlockCipher cipher;

   public OFBBlockCipher(BlockCipher cipher, int blockSize) {
      this.cipher = cipher;
      this.blockSize = blockSize / 8;
      this.IV = new byte[cipher.getBlockSize()];
      this.ofbV = new byte[cipher.getBlockSize()];
      this.ofbOutV = new byte[cipher.getBlockSize()];
   }

   public BlockCipher getUnderlyingCipher() {
      return this.cipher;
   }

   public void init(boolean encrypting, CipherParameters params) {
      if (params instanceof ParametersWithIV) {
         byte[] var3;
         ParametersWithIV encrypting1;
         byte[] params1;
         if ((params1 = (encrypting1 = (ParametersWithIV)params).getIV()).length < (var3 = this.IV).length) {
            int var4 = var3.length - params1.length;
            int var5 = params1.length;
            System.arraycopy(params1, 0, var3, var4, var5);

            byte[] var10;
            for(int var9 = 0; var9 < (var10 = this.IV).length - params1.length; ++var9) {
               var10[var9] = 0;
            }
         } else {
            int params2 = var3.length;
            System.arraycopy(params1, 0, var3, 0, params2);
         }

         this.reset();
         this.cipher.init(true, encrypting1.getParameters());
      } else {
         this.reset();
         this.cipher.init(true, params);
      }

   }

   public String getAlgorithmName() {
      return this.cipher.getAlgorithmName() + "/OFB" + this.blockSize * 8;
   }

   public int getBlockSize() {
      return this.blockSize;
   }

   public int processBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int var5;
      if (inOff + (var5 = this.blockSize) > in.length) {
         throw new DataLengthException("input buffer too short");
      } else if (outOff + var5 > out.length) {
         throw new DataLengthException("output buffer too short");
      } else {
         byte[] var8 = this.ofbV;
         byte[] var6 = this.ofbOutV;
         this.cipher.processBlock(var8, 0, var6, 0);

         int var9;
         for(var5 = 0; var5 < (var9 = this.blockSize); ++var5) {
            out[outOff + var5] = (byte)(this.ofbOutV[var5] ^ in[inOff + var5]);
         }

         byte[] var10002 = in = this.ofbV;
         inOff = in.length - var9;
         System.arraycopy(var10002, var9, in, 0, inOff);
         int var7;
         inOff = (in = this.ofbV).length - (var7 = this.blockSize);
         System.arraycopy(this.ofbOutV, 0, in, inOff, var7);
         return this.blockSize;
      }
   }

   public void reset() {
      OFBBlockCipher var10000 = this;
      byte[] var10001 = this.IV;
      byte[] var2 = this.ofbV;
      int var1 = var10001.length;
      System.arraycopy(var10001, 0, var2, 0, var1);
      var10000.cipher.reset();
   }
}
